#!/usr/bin/perl 
# gettext.pl
#  Given an XML file, produce all of the text in the descendants of selected
# elements.
#  Part of the CTAN search package.  INtended to be searched by 
# CTANcataloguesearch.pl
# HISTORY
# 1999-Nov-08 Jim Hefferon tex@joshua.smcvt.edu  Written.
use XML::DOM;
use XML::XQL;
use XML::XQL::DOM;

# read the command line arguments
use Getopt::Long;
$opt_help=0;
$opt_ignore_default_elements=0; 
$opt_ignore_default_parenthesized_elements=0;
$opt_ignore_default_bracketed_elements=0;
unless (GetOptions("help",
                    "input-file=s",
                    "output-file=s",
                    "ignore-default-elements",
	            "ignore-default-parenthesized-elements",
	            "ignore-default-bracketed-elements",
                    "element=s@",
                    "parenthesized-element=s@",
                    "bracketed-element=s@")) {
  die "unable to get options: $!";
}

# help message
my $Usage=<<'End_Usage';
  tryxql --input-file=<filename> [--output-file=<filename> (appended)]
         [--element] [--parenthesized-element] [--bracketed-element] 
         [--ignore-default-elements]
         [--ignore-default-parenthesized-elements] 
         [--ignore-default-bracketed-elements]
Given an XML file, the result is all of the text in the descendants of the 
selected elements, or that text between parentheses (((( .. )))), or 
between brackets <<<< .. >>>>.
End_Usage

if ($opt_help) {
    print $Usage;
    exit;
}
if ($opt_input_file eq "") {
    die "no input file specified"; 
}
if ($opt_output_file eq "") {
    open(OUTFILE,">-") 
        or die "cannot open output $opt_output_file: $!";
} else {
    open(OUTFILE,">>$opt_output_file") 
        or die "cannot open output file $opt_output_file: $!";
}

# lists of elets to query on
#   elements to just print the results on
@default_elet_list=("/entry/about/name","/entry/description/abstract","/entry/about/author/name");
#   elements to print the results on in parentheses
@default_pelet_list=("/entry/distribution/ctan");
#   elements to print the results on in brackets
@default_belet_list=("/entry/about/caption");

if ($opt_ignore_default_elements) {
    @elet_list=@opt_element;
} else {
    @elet_list=(@default_elet_list,@opt_element);
}
if ($opt_ignore_default_parenthesized_elements) {
    @pelet_list=@opt_parenthesized_element;
} else {
    @pelet_list=(@default_pelet_list,@opt_parenthesized_element);
}
if ($opt_ignore_default_bracketed_elements) {
    @belet_list=@opt_bracketed_element;
} else {
    @belet_list=(@default_belet_list,@opt_bracketed_element);
}


# ----Done handling the command line arguments----  


# Parse the .xml file and do the queries. 
$parser = new XML::DOM::Parser;
$doc = $parser->parsefile ("$opt_input_file") 
         || die "cannot parse (open?) input file $opt_input_file: $!";

$elet_str = doqueries("","",@elet_list);
$pelet_str = doqueries("((((","))))",@pelet_list);
$belet_str = doqueries("<<<<",">>>>",@belet_list);
$total_str = "$elet_str $pelet_str $belet_str \n"; # add the one newline

#open(OUTFILE,">>$opt_output_file") 
#        or die "cannot open output file $opt_output_file: $!";
print OUTFILE $total_str;


#.........subroutines.............

# run the XQL queries
#  arg 1 is the prefix that will be put on the found strings (maybe "((((").
#  arg 2 is the postfix (maybe "))))")
#  the rest of the arguments make the array of query strings
sub doqueries {
    my $leftboundary = shift;
    my $rightboundary = shift;
    my @query_list = @_;
    my $returned_str = "";

    for $query_expr (@query_list) {
        $query = new XML::XQL::Query (Expr => $query_expr, @options);
        @result = $query->solve ($doc);

        for my $elem (@result) {
            $returned_str = $returned_str.alltextchildren($elem);
        }
    }
    $returned_str =~ s/^\s*//; #strip leading spaces
    $returned_str =~ s/\s*$//; #strip trailing spaces
    $returned_str =~ s/\n//g; # strip out newlines
    # return boundaries, even if $returned_str is empty?  Guess so.
    return ("$leftboundary$returned_str$rightboundary")
}


# Gather text from all TEXT_NODE descendants (recursively)
#  arg1  a node
sub alltextchildren {
    my $elem = shift;
    my $childnode;
    my $collected_text="";

    if ($elem->getNodeType == TEXT_NODE) {
        $collected_text = $elem->toString;
    }
    for $childnode ($elem->getChildNodes) {
        $collected_text = $collected_text." ".alltextchildren($childnode);
    }
    return($collected_text);
}


