#!/usr/bin/perl
# fetch.pl
#  Using names of mirrors and prefix paths to the top of their archive tree 
# read from an external file, see which mirrors respond to a FTP request to 
# get various files (typically, with .zip or a .tar.gz extensions).
#
#  2000-Jan-13 Jim Hefferon tex@joshua.smcvt.edu 
use Net::FTP;
use Getopt::Long;

GetOptions(
	   "help" => \$help,
	   "verbose" => \$verbose,
	   "sitelist=s" => \$sitelist,
	   "resultlist=s" => \$resultlist,
	   "file=s" => \$file_to_fetch,
	   "localfile=s" => \$localfile,
	   "extension=s@" => \@extensions,
	   "username=s" => \$username,
	   "password=s" => \$password,
	   "timeout-connect=i" => \$timeout_connect,
	   "timeout-login=i" => \$timeout_login,
	   "timeout-getfile=i" => \$timeout_getfile,
	   "timeout-quit=i" => \$timeout_quit,
	   "debug" => \$debug,
	   "reckless" => \$reckless,  # ignore atypical interrupts (Ctl-C stuff?)
	   "fake" => \$fake
	   ) or die "$0: Cannot parse command line; try $0 --help\n";
# defaults
$sitelist ||= "mirrors.txt";
$resultlist ||= "responding_mirrors.txt";
$file_to_fetch ||= "macros/latex/contrib/supported/beton";
$localfile ||= "beton.test";
if (!(@extensions)) {
    @extensions = ("zip","tar.gz");
}
$username ||= "ftp";
$password ||= "ftpmaint\@tug.ctan.org"; # escape the @
$timeout_connect ||= 200; # in seconds
$timeout_login ||= 200; 
$timeout_getfile ||= 200; 
$timeout_quit ||= 200; 

if ($debug) {
    print STDERR "$0: debug values";
    print STDERR " \$sitelist=$sitelist, \$resultlist=$resultlist \n";
    print STDERR " \$file_to_fetch=$file_to_fetch, \$localfile=$localfile \n";
    print STDERR " \@extensions=@extensions \n";
    print STDERR " \$username=$username, \$password=$password \n";
    print STDERR " timeouts: connect=$timeout_connect, login=$timeout_login,  getfile=$timeout_getfile, quit=$timeout_quit \n";
}

if ($help) {
    print "$0: FTP a file from a number of sites. Options: \n";
    print "  --help and --verbose, for more talk \n";
    print "  --sitelist=<fn>: where to get input list of sites \n";
    print "  --resultlist=<fn>: where to put list of successful sites \n";
    print "  --file=<fn>, --localfile=<fn>: what to get, where to put it \n";
    print "  --extension=<str>: extension on file (no .; use multiple times) \n";
    print "  --username=<str>, --password=<str>: how to login \n";
    print "  --timeout-connect=<int>, --timeout-login=<int>,\n";
    print "    --timeout-getfile=<int>, --timeout-quit=<int>: in seconds \n";
    print " (and --debug, --fake, and --reckless for development work) \n";

    exit 0;
}

if ($fake) {
    print STDERR "No processing done as --fake option is on.\n";
    exit 0;
}

####### main script ##############################
# read the list of mirrors
open(MIRRORLIST, "< $sitelist")
    or die "could not open $mirrorlistfn: $!\n";
$resultsfn="responding_mirrors.txt";
open(RESULTS,"> $resultlist")
    or die "could not open $resultlist: $!\n";


MAINLOOP: while (<MIRRORLIST>) {
    next if (/^\#/); # comments are lines starting with a #
    @fields=split;   
    $hostname=shift(@fields); # lines have three main fields; name, path, geo 
    $prefix=shift(@fields);   # path goes to top of ftp's archive.
    $geo=join ' ', @fields;  #includes open and close parens
    $result_report="$hostname $prefix $geo ";  
    print "$result_report\n";
    # the following operations each have a timeout time, Programming Perl p 341
    eval { # CONNECT set up a timeout facility
        local $SIG{ALRM} = sub { die "skip this host" };
	alarm($timeout_connect);
	$ftp = Net::FTP->new("$hostname",
			     Debug => $verbose);
	alarm(0);
    };
    if (!($ftp)) {
	if ($debug or $verbose) {
	    print STDERR "trouble with $hostname: no connection\n";
	}
	next MAINLOOP;
    }
    if ($@) {
	if ($@ =~ /skip this host/) {
	    if ($debug or $verbose) {
		print STDERR "trouble with $hostname: connect timeout\n";
	    }
	} elsif ($@ =~ /Bad arg length for Socket::unpack_sockaddr_in/) {
	    if ($debug or $verbose) {
		print STDERR "trouble connecting with $hostname: $@\n";
	    }
	} else {
	    unless ($reckless) {
		die "$0: atypical signal during connect with $hostname: $@\n";
	    }
	}
	next MAINLOOP;
    }

    eval { # LOG IN: set up a timeout facility 
        local $SIG{ALRM} = sub { die "skip this host" };
	alarm($timeout_login);
	$ftp->login($username,$password);
	alarm(0);
    };
    if ($@) {
	if ($@ =~ /skip this host/) {
	    if ($debug or $verbose) {
		print STDERR "trouble with $hostname: login timeout\n";
	    }
	} elsif ($@ =~ /Bad arg length for Socket::unpack_sockaddr_in/) {
	    if ($debug or $verbose) {
		print STDERR "trouble logging in with $hostname: $@\n";
	    }
	} else {
	    unless ($reckless) {
		die "$0: atypical signal during login with $hostname: $@\n";
	    }
	}
	next MAINLOOP;
    }

    eval { # GET THE FILE(s) set up a timeout facility 
        local $SIG{ALRM} = sub { die "skip this host" };
	alarm($timeout_getfile);
	$ftp->binary();  # binary mode
	foreach $extension (@extensions) {
	    next unless ($ftp->get("$prefix/$file_to_fetch.$extension"));
	    $result_report="$result_report $extension";
	}
	alarm(0);
    };
    if ($@) {
	if ($@ =~ /skip this host/) {
	    if ($debug or $verbose) {
		print STDERR "trouble with $hostname: get file timeout\n";
	    }
	} elsif ($@ =~ /Bad arg length for Socket::unpack_sockaddr_in/) {
	    if ($debug or $verbose) {
		print STDERR "trouble getting file with $hostname: $@\n";
	    }
	} else {
	    unless ($reckless) {
		die "$0: atypical signal getting file from $hostname: $@\n";
	    }
	}
	next MAINLOOP;
    }

    eval { # LOG OUT set up a timeout facility
        local $SIG{ALRM} = sub { die "skip this host" };
	alarm($timeout_quit);
	$ftp->quit();
	alarm(0);
    };
    if ($@) {
	if ($@ =~ /skip this host/) {
	    if ($debug or $verbose) {
		print STDERR "trouble with $hostname: quit timeout\n";
	    }
	} elsif ($@ =~ /Bad arg length for Socket::unpack_sockaddr_in/) {
	    if ($debug or $verbose) {
		print STDERR "trouble with quitting $hostname: $@\n";
	    }
	} else {
	    unless ($reckless) {
		die "$0: atypical signal during quiting $hostname: $@\n";
	    }
	}
	next MAINLOOP;
    }
    print RESULTS "$result_report\n";
}


close(MIRRORLIST) or die "can not close $sitelist: $!\n";
close(RESULTS) or die "can not close $resultlist: $!\n";


