#!/usr/bin/perl 
# dir_info.pl
#  Build a perl module that can be asked for information about CTAN
# directories.
# HISTORY
# 2000-Dec-21 Jim Hefferon tex@joshua.smcvt.edu  Written.
# 2001-Mar-31 JH Fixed bug in distinction between Graham's package name and
#               id.

# read the command line arguments
use Getopt::Long;

# The option names and the default values
$optstr_help = "help";
$opt_help = 0;
$optstr_verbose = "verbose";
$opt_verbose = 0;
$optstr_catdir = "catalogue-entry-directory";
$opt_catalogue_entry_directory = "/home/ftp/pub/tex-archive/help/Catalogue/entries";
$optstr_outpm = "output-pm-file";
$opt_output_pm_file = "DirInfo.pm";
$optstr_extinfo = "extra-info-file";
$opt_extra_info_file = "extra_info"; #needs to have form: dir \t info
$optstr_direlets = "xml-directory-entries";
@opt_xml_directory_entries = ("/entry/distribution/ctan");
$optstr_id = "xml-id";
$opt_xml_id = "/entry/\@id";
$optstr_nameelets = "xml-name-entries";
@opt_xml_name_entries = ("/entry/about/name");
$optstr_abselets = "xml-abstract-entries";
@opt_xml_abstract_entries = ("/entry/description/abstract");
$optstr_captelets = "xml-caption-entries";
@opt_xml_caption_entries = ("/entry/about/caption");
unless (GetOptions(
	   "$optstr_help" => \$opt_help,
	   "$optstr_verbose" => \$opt_verbose,
	   "$optstr_catdir=s" => \$opt_catalogue_entry_directory,
	   "$optstr_outpm=s" => \$opt_output_pm_file,
	   "$optstr_extinfo=s" => \$opt_extra_info_file,
	   "$optstr_direlets=s@" => \@opt_xml_directory_entries,
	   "$optstr_id=s" => \$opt_xml_id, 
	   "$optstr_nameelets=s@" => \@opt_xml_name_entries,
	   "$optstr_abselets=s@" => \@opt_xml_abstract_entries,
	   "$optstr_captelets=s@" => \@opt_xml_caption_entries
	   )) {
    die "unable to get options: $!";
} 

my $Usage=<<"End_Usage";
$0: [--$optstr_help] 
  [--$optstr_verbose]
  [--$optstr_catdir] directory of Catalogue .xml entries; default: $opt_catalogue_entry_directory
  [--$optstr_extinfo] name of extra info file: $opt_extra_info_file
  [--$optstr_outpm] name of output perl module: $opt_output_pm_file
  [--$optstr_direlets] array of xml elements giving directories: (@opt_xml_directory_entries)
  [--$optstr_id] package id attribute: $opt_xml_id
  [--$optstr_nameelets] array of xml elements giving package name: (@opt_xml_name_entries)
  [--$optstr_abselets] array of xml elements giving abstract text: (@opt_xml_abstract_entries)
  [--$optstr_captelets] array of xml element giving caption text: (@opt_xml_caption_entries)
Extract things from the Catalogue and from an extra info file into hash 
keyed as indicated (default: the CTAN paths).  This hash is used 
for searching, autoindexing, etc.  The hash is accessed by using the output 
perl module.
End_Usage

if ($opt_verbose) {
    print "$0 running with these options .. \n",
      " .. $optstr_help => $opt_help\n",
      " .. $optstr_verbose => $opt_verbose\n",
      " .. $optstr_catdir => $opt_catalogue_entry_directory\n",
      " .. $optstr_outpm => $opt_output_pm_file\n",
      " .. $optstr_extinfo => $opt_extra_info_file\n",
      " .. $optstr_direlets => (@opt_xml_directory_entries)\n",
      " .. $optstr_id] => $opt_xml_id",
      " .. $optstr_nameelets => (@opt_xml_name_entries)\n",
      " .. $optstr_abselets => (@opt_xml_abstract_entries)\n",
      " .. $optstr_captelets => (@opt_xml_caption_entries)\n";
}
if ($opt_help) {
    print $Usage;
    exit;
}
# ----Done handling the command line arguments----  

# open the .pm file
open OUT, ">$opt_output_pm_file"
    or die "$0: cannot open $opt_output_pm_file: $!";


# print the .pm file's header
print OUT <<"END_of_Module_Header";
#package DirInfo;  # FOR TESTING ONLY; Next line for production
package Apache::DirInfo;
# file Apache/DirInfo.pm 
#  Information about CTAN directories 
#  !!! generated from dir_info.pl; don't edit this file directly !!! 
use Exporter;
\@ISA = (\'Exporter\');
\@EXPORT = qw( 
	       \%descs 
	       \&new_key 
	       \&insert_cat_info \&get_cat_info  \&get_cat 
	       \&get_extra_info
	       \&any_info \&display_info
	     );
use Apache::CTAN;

END_of_Module_Header


# print out to the .pm file it's data accessor functions 
print(OUT <<"END_of_Module_Fcns");
 
# The descs hash
#  Structure: key => [{..},[..]]
# The keys are CTAN directories.  The vals are pairs.  Entry 0 is
# a hash of Catalogue package_name=>description pairs.  Entry 1 is
# a list of strings that are further descriptions.
\%descs = (
	  );


# new_key: put a new key into the descs hash, with a null value
# ARGS
#  $k  the key; a CTAN directory  
sub new_key {
    my \$k = shift();

    \$descs{\$k} = [{},[]];
}


# insert_cat_info: put a catalogue k=>v pair into the descs entry
# ARGS
#  \$dkey  key into descs; the directory
#  \$catkey, \$val  key=>val pair to be entered
sub insert_cat_info {
    my \$dkey = shift();
    my \$catkey = shift();
    my \$val = shift();

    unless (exists(\$descs{\$dkey})) {
	new_key(\$dkey);
    }
    \@{ \$descs{\$dkey} }->[0]{\$catkey} = \$val;
}


# get_cat_info:  get val associated with a cat key for a desc key
# ARGS
#  \$dkey  key into descs hash; a directory name
#  \$catkey  a key for the Catalogue (CTAN package name)
# RETURNS
#  undef if no such $dkey
#  "" if no such $catkey
#  val o.w.
sub get_cat_info {
    my \$dkey = shift ();
    my \$catkey = shift();

    if (exists(\$descs{\$dkey})) {
	if (exists(\@{ \$descs{\$dkey} }->[0]{\$catkey})) {
	    return \@{ \$descs{\$dkey} }->[0]{\$catkey};
	} else {
	    return "";
	}
    } else {
	return undef;
    }
}

# get_cat_info:  get cat hash associated with a desc key
# ARGS
#  \$dkey  key into descs hash; a directory name
# RETURNS
#  undef if no such $dkey
#  hash o.w.
sub get_cat {
    my \$dkey = shift ();

    if (exists(\$descs{\$dkey})) {
	return %{ \@{ \$descs{\$dkey} }->[0] }; 
    } else {
	return undef;
    }
}


# trim: remove surrounding whitespace 
#  from Perl Cookbook, p. 30
sub trim {
    my \@out = \@_;

    for (\@out) {
	s/^\\s+//;
	s/\\s+\$//;
    }
    return wantarray ? \@out : \$out[0];
}





# insert_extra_info: put a entry into the descs entry
# ARGS
#  $dkey  key into descs; the directory
#  $info  info to be entered
sub insert_extra_info {
    my \$dkey = shift();
    my \$info = shift();

    unless (exists(\$descs{\$dkey})) {
	new_key(\$dkey);
    }
    push  \@{ \@{ \$descs{\$dkey} }[1] }, \$info;
}



# get_extra_info:  get extra info associated with a desc key
# ARGS
#  $dkey  key into descs hash; a directory name
# RETURNS
#  undef if no such $dkey
#  array o.w.
sub get_extra_info {
    my \$dkey = shift ();

    if (exists(\$descs{\$dkey})) {
	return \@{ \@{ \$descs{\$dkey} }[1] };
    } else {
	return undef;
    }
}


# any_info
#  return true if any info is available on the directory
# ARGS
#  \$dir_key Name of directory
# RETURN
#  true or false
sub any_info {
    my \$dir_key = shift();
    \$dir_key =~ s!/\$!!; # if it ends in / get rid of that

    if (exists(\$descs{\$dir_key})) {
	return (get_cat(\$dir_key) || get_extra_info(\$dir_key));
    } else {
	return 0;
    }
} 


# display info
#  show the information about a directory
# ARGS
#  \$dir_key  Name of directory
# RETURN
#  string for displaying information
sub display_info {
    my \$dir_key = shift();
    my \$URL_str = shift();

    \$dir_key =~ s!/\$!!; # if it ends in / get rid of that

    my \$k = ""; my \$v = "";
    my \$returned_str = "";

    my \@extra_info = get_extra_info(\$dir_key);
    if (\@extra_info) {
	\$returned_str = \$returned_str."General information on this material\\n";
	\$returned_str = \$returned_str."<ul>\\n";
	foreach \$v (\@extra_info) {
	    \$returned_str = \$returned_str."<li> \$v \\n";
	}
	\$returned_str = \$returned_str."</ul>\\n";
    }

    my \%cat_hash = get_cat(\$dir_key);
    if (\%cat_hash) {
	\$returned_str = \$returned_str."Catalogue information\\n";
	\$returned_str = \$returned_str."<ul>\\n";
	while ((\$k,\$v) = each(\%cat_hash)) {
            \$v =~ /(\\s*)<<<<(.*)>>>>(.*)/;
	    \$pkg_name = trim(\$2);
	    \$pkg_desc = trim(\$3);
	    \$returned_str = \$returned_str."<li><a href=\\"\".WEBFTP_ROOT().\"help/Catalogue/entries/\$k.html\$URL_str\\"><b>\$pkg_name</b></a> \$pkg_desc\\n";
	}
	\$returned_str = \$returned_str."</ul>\\n";
    }

    return "<p>".\$returned_str."</p>";
}




END_of_Module_Fcns



# print out to the .pm file the initializer routine that sucks info in
# from below it's __DATA__ line
print(OUT <<"END_of_Module_Body");
\@line_fields = ();
while (\$line = <DATA>) {
    \@line_fields = split /\\t/, \$line;
    \$dirkey = shift(\@line_fields);
    \$dirkey = trim(\$dirkey);
    \$line_fields = \@line_fields;
    if (\$line_fields == 1) {
	\$extraval = trim(shift(\@line_fields));
	insert_extra_info(\$dirkey,\$extraval);
    } elsif (\$line_fields >= 2) {
	\$catkey = trim(shift(\@line_fields));
	\$catname = trim(shift(\@line_fields));
	\$cattxt = trim(shift(\@line_fields));
	insert_cat_info(\$dirkey,\$catkey,\"\$catname \$cattxt\");
    } else {
	# ignore it
    }
}


END_of_Module_Body


print OUT "1;\n\n__DATA__\n"; # .pm's must return true

# With  the .pm file's header and data fcns in place, the only thing left
# is to put in the data.
#  This data is tab-separated.  Catalogue information come in with three
# fields (CTAN directory\tpackage_name\tpackage description or abstract).
# The extra information comes in with two fields (dir\textra info). 

# First stock the data structure with information from the Catalogue.
opendir(DIR,$opt_catalogue_entry_directory) 
    or die "cannot open $opt_catalogue_entry_directory: $!";

# Parse the .xml files and do the queries. 
use XML::DOM;
use XML::XQL;
use XML::XQL::DOM;
$parser = new XML::DOM::Parser;

while (defined($xmlfile = readdir(DIR))) {
    next unless ($xmlfile =~ /\.xml$/);
    $doc = $parser->parsefile ("$opt_catalogue_entry_directory/$xmlfile") 
           or die "cannot parse (open?) input file $opt_catalogue_entry_directory/$xmlfile: $!";

    $key_str = doqueries(@opt_xml_directory_entries);
    @id_array=($opt_xml_id);
    $id_str = doqueries(@id_array);
    $abstr_str = doqueries(@opt_xml_abstract_entries);
    $capt_str = doqueries(@opt_xml_caption_entries);
    $name_str = doqueries(@opt_xml_name_entries);
    unless (($key_str eq "") || ($key_str eq 'unknown')) {
	if ($capt_str) {
	    $description = $capt_str;
	} else {
	    $description = $abstr_str;
	}
	print OUT $key_str
	              ."\t".$id_str
	              ."\t"."<<<<".$name_str.">>>>"
		      ."\t".$description."\n";
    }
}

closedir(DIR) 
    or die "cannot close $opt_catalogue_entry_directory: $!";


# Second, stock the data structure with extra information
$extra_info_file = "extra_info"; #needs to have form: dir \t info
open EXTRA, "$extra_info_file"
    or die "$0: cannot open $extra_info_file: $!";
while ($line = <EXTRA>) {
    print OUT $line;
}
close(EXTRA)
    or die "cannot close $optstr_extinfo: $!";


#.........subroutines for XQL.............

# doqueries
#   run the XQL queries; get all text children of an array of nodes
# ARGS  
#  an array of query strings
# RETURNS
#  a string
sub doqueries {
    my @query_list = @_;
    my $returned_str = "";

#print "\n About to run doqueries: ",@_;
    for $query_expr (@query_list) {
        $query = new XML::XQL::Query (Expr => $query_expr, @options);
        @result = $query->solve ($doc);

        for my $elem (@result) {
            $returned_str = $returned_str.alltextchildren($elem);
        }
    }
    $returned_str =~ s/^\s*//; #strip leading spaces
    $returned_str =~ s/\s*$//; #strip trailing spaces
    $returned_str =~ s/\n//g; # strip out newlines
    return ("$returned_str")
}


# Gather text from all TEXT_NODE descendants (recursively)
# ARGS
#  arg1  a node
# RETURNS
#  a string
sub alltextchildren {
    my $elem = shift;
    my $childnode;
    my $collected_text="";

    if ($elem->getNodeType == TEXT_NODE) {
        $collected_text = $elem->toString;
    }
    for $childnode ($elem->getChildNodes) {
        $collected_text = $collected_text." ".alltextchildren($childnode);
    }
    return($collected_text);
}

# I used this to test the DirInfo.pm module
#  CAREFUL!! change the first line so it doesn't say `Apache::'
# test_dir_info.pl
#-------------------------------------------
##!/usr/bin/perl
#use DirInfo;
#
##print "descs is:",%descs,"\n\n";
#foreach $key (keys(%descs)) {
#    %cat_hash = get_cat($key);
#    if (%cat_hash) {
#	foreach $ck (keys(%cat_hash)) {
#	    print "test: info is: $key => $ck => ",get_cat_info($key,$ck),"\n";
#	}
#    }
#    @extra_info = get_extra_info($key);
#    if (@extra_info) {
#	print "test: extra info is: $key ... @extra_info\n";
#    }
#    print "\n",display_info($key),"\n";
#print "Finally, .. a test of whether dirs have entries.\n";
#foreach $key (("macros/lotex/contrib", "macros/latex/contrib/supported/shadethm")) {
#    print " any_info on $key?";
#    if (any_info($key)) {
#	print " yes";
#    } else {
#	print " no";
#    }
#    print "\n";
#}
